@@ -4,16 +4,15 @@ from rnaglib.transforms import GraphRepresentation
 
 ta = ChemicalModification(
     root="RNA_CM",
-    recompute=True,
-    debug=True,
+    recompute=False,
+    debug=False,
 )
 
-
 # Add representation
 ta.dataset.add_representation(GraphRepresentation(framework="pyg"))
 
 # Splitting dataset
-ta.get_split_loaders(recompute=True)
+ta.get_split_loaders(recompute=False)
 
 # Train model
 # Either by hand:
