@@ -1,6 +1,6 @@
+from rnaglib.learning.task_models import PygModel
 from rnaglib.tasks import InverseFolding
 from rnaglib.transforms import GraphRepresentation
-from rnaglib.learning.task_models import PygModel
 
 ta = InverseFolding(root="RNA_IF", recompute=True, debug=False)
 
@@ -12,8 +12,8 @@ ta.get_split_loaders(recompute=True)
 # Train model
 model = PygModel(
     num_node_features=ta.metadata["description"]["num_node_features"],
-    num_classes=ta.metadata["description"]["num_classes"],
-    graph_level=False
+    num_classes=ta.metadata["description"]["num_classes"] + 1,  # to account for non standard nucs
+    graph_level=False,
 )
 model.configure_training(learning_rate=0.001)
 model.train_model(ta, epochs=10)
