@@ -3,7 +3,7 @@ from rnaglib.transforms import GraphRepresentation
 from rnaglib.learning.task_models import PygModel
 
 # Creating task
-ta = BenchmarkBindingSite(root="RNA_Site_Bench", recompute=True, debug=True)
+ta = BenchmarkBindingSite(root="RNA_Site_Bench", recompute=True)
 
 # Add representation
 ta.dataset.add_representation(GraphRepresentation(framework="pyg"))
