@@ -1,21 +1,30 @@
 """Abstract task classes"""
 
+import tarfile
+import shutil
 from collections.abc import Sequence
 from functools import cached_property
 import hashlib
 import json
+
 import numpy as np
+import pandas as pd
 import os
+from typing import Union, Literal, Optional
 from pathlib import Path
-from sklearn.metrics import accuracy_score, f1_score, matthews_corrcoef, roc_auc_score
+from sklearn.metrics import accuracy_score, f1_score, matthews_corrcoef, roc_auc_score, jaccard_score, balanced_accuracy_score
 import torch
 from torch.utils.data import DataLoader
 import tqdm
 
 from rnaglib.data_loading import Collater, RNADataset
-from rnaglib.transforms import FeaturesComputer, SizeFilter
-from rnaglib.utils import DummyGraphModel, DummyResidueModel, dump_json, tonumpy
-from rnaglib.dataset_transforms import RandomSplitter, Splitter, DistanceComputer, RedundancyRemover
+from rnaglib.transforms import Transform, FeaturesComputer, SizeFilter
+from rnaglib.utils import DummyGraphModel, DummyResidueModel, dump_json, tonumpy, download
+from rnaglib.dataset_transforms import RandomSplitter, Splitter, RedundancyRemover
+from rnaglib.dataset_transforms import StructureDistanceComputer, CDHitComputer
+
+ZENOD_RECORD = "182660"
+ZENODO_URL = f"https://sandbox.zenodo.org/records/{ZENOD_RECORD}/files/"
 
 
 class Task:
@@ -29,60 +38,86 @@ class Task:
     :param recompute: whether to recompute the task info from scratch or use what is stored in root.
     :param splitter: rnaglib.dataset_transforms.Splitter object that handles splitting of data into train/val/test indices.
     If None uses task's default_splitter() attribute.
+    :param debug: if True, loads a smaller version of the data for fast
+    testing.
+    :param save: if True, saves the task data to root directory.
+    :param in_memory: if True, data is loaded from memory instead of on disk.
+    :param size_thresholds: 2 element list with lower and upper bound on RNA
+    size.
+    :param precomputed: if True, tries to download processed task from Zenodo.
+    :param additional_metadata: dictionary with metadata to include in task.
     """
 
     def __init__(
-            self,
-            root: str | os.PathLike,
-            recompute: bool = False,
-            splitter: Splitter = None,
-            debug: bool = False,
-            save: bool = True,
-            in_memory: bool = False,
-            size_thresholds: Sequence = None,
-            multi_label=False,
-            distance_computers: DistanceComputer | list[DistanceComputer] = (),
-            redundancy_remover: RedundancyRemover = None,
+        self,
+        root: Union[str, os.PathLike],
+        recompute: bool = False,
+        splitter: Optional[Splitter] = None,
+        debug: bool = False,
+        save: bool = True,
+        in_memory: bool = False,
+        size_thresholds: Optional[Sequence] = None,
+        precomputed=True,
+        additional_metadata=None
     ):
         self.root = root
         self.dataset_path = os.path.join(self.root, "dataset")
         self.recompute = recompute
+        self.precomputed = precomputed
         self.debug = debug
         self.save = save
         self.in_memory = in_memory
         self.size_thresholds = size_thresholds
-        self.multi_label = multi_label
-        self.distance_computers = [distance_computers] if not isinstance(distance_computers,
-            Sequence) else distance_computers
-        self.redundancy_remover = redundancy_remover
 
-        self.metadata = self.init_metadata()
+        self.init_metadata(additional_metadata=additional_metadata)
 
         # Load or create dataset
-        if not os.path.exists(self.dataset_path) or recompute:
+        if self.precomputed and not os.path.exists(Path(self.root) / "done.txt"):
+            try:
+                self.from_zenodo()
+            except Exception as e:
+                print(f"Error downloading dataset from \
+                Zenodo: {e}. Check if the dataset is \
+                available at zenodo, otherwise use `precomputed=False` to build locally.")
+
+        elif not os.path.exists(Path(self.root) / "done.txt") or recompute:
+            os.makedirs(self.dataset_path, exist_ok=True)
             print(">>> Creating task dataset from scratch...")
             # instantiate the Size filter if required
             if self.size_thresholds is not None:
                 self.size_filter = SizeFilter(size_thresholds[0], size_thresholds[1])
             self.dataset = self.process()
+            self.dataset.features_computer = self.get_task_vars()
+
+            self.metadata.update(self.describe())
+            self.post_process()
+            self.metadata["data_version"] = self.dataset.version
         else:
-            # self.dataset, self.metadata, (self.train_ind, self.val_ind, self.test_ind) = self.load()
             self.load()
 
-        self.dataset.features_computer = self.get_task_vars()
-
         # Set splitter after dataset is available
-        self.splitter = self.default_splitter if splitter is None else splitter
-
         # Split dataset if it wasn't loaded from file
+        self.splitter = self.default_splitter if splitter is None else splitter
         if not hasattr(self, "train_ind"):
+            print("no split found, splitting")
             self.split(self.dataset)
 
         if self.save:
             self.write()
+            with open(Path(self.root) / "done.txt", "w") as f:
+                f.write("")
 
-        # compute metadata
-        self.describe()
+
+    def from_zenodo(self):
+        """Downloads the task dataset from Zenodo and loads it."""
+
+        print(">>> Downloading task dataset from Zenodo...")
+        url = ZENODO_URL + f"{self.name}.tar.gz"
+        download(url)
+        with tarfile.open(f"{self.name}.tar.gz") as tar_file:
+            tar_file.extractall()
+            shutil.move(f"{self.name}", self.root)
+        self.load()
 
     def process(self) -> RNADataset:
         """Tasks must implement this method.
@@ -93,9 +128,11 @@ class Task:
         """
         raise NotImplementedError
 
-    def init_metadata(self) -> dict:
-        """Optionally adds some key/value pairs to self.metadata."""
-        return {}
+    def init_metadata(self, additional_metadata: Optional[dict]= None) -> None:
+        """Initialize dictionary to hold key/value pairs to self.metadata."""
+        self.metadata = {}
+        if not additional_metadata is None:
+            self.metadata.update(additional_metadata)
 
     def get_task_vars(self) -> FeaturesComputer:
         """Define a FeaturesComputer object to set which input and output variables will be used in the task."""
@@ -106,8 +143,34 @@ class Task:
         """The splitter used if no other splitter is specified."""
         return RandomSplitter()
 
-    def split(self, dataset):
-        """Calls the splitter and returns train, val, test splits."""
+    def post_process(self):
+        """
+        The most common post_processing steps to remove redundancy.
+
+        Other tasks should implement their own if this is not the desired default behavior
+        """
+        cd_hit_computer = CDHitComputer(similarity_threshold=0.9)
+        cd_hit_rr = RedundancyRemover(distance_name="cd_hit", threshold=0.9)
+        self.dataset = cd_hit_computer(self.dataset)
+        self.dataset = cd_hit_rr(self.dataset)
+
+        us_align_computer = StructureDistanceComputer(name="USalign")
+        us_align_rr = RedundancyRemover(distance_name="USalign", threshold=0.8)
+        self.dataset = us_align_computer(self.dataset)
+        self.dataset = us_align_rr(self.dataset)
+
+        # PATCH: delete graphs from dataset/ lost during redundancy removal
+        for f in os.listdir(self.dataset.dataset_path):
+            if Path(f).stem not in self.dataset.all_rnas:
+                os.remove(Path(self.dataset.dataset_path) / f)
+
+        self.dataset.save_distances()
+
+    def split(self, dataset: RNADataset):
+        """Calls the splitter and returns train, val, test splits.
+
+        :param dataset: 
+        """
         splits = self.splitter(dataset)
         self.train_ind, self.val_ind, self.test_ind = splits
         return splits
@@ -150,7 +213,25 @@ class Task:
             print(">>> Done")
         return self.train_dataset, self.val_dataset, self.test_dataset
 
-    def get_split_loaders(self, recompute=True, **dataloader_kwargs):
+    def add_representation(self, representation: Transform):
+        self.dataset.add_representation(representation)
+        pass
+
+    def remove_representation(self, representation_name: str):
+        self.dataset.remove_representation(representation_name)
+        pass
+
+    def add_feature(
+        self,
+        feature: Union[str, Transform],
+        feature_level: Literal["residue", "rna"] = "residue",
+        is_input: bool = True,
+    ):
+        """Shortcut to RNADataset.add_feature"""
+        self.dataset.add_feature(feature=feature, feature_level=feature_level, is_input=is_input)
+        pass
+
+    def get_split_loaders(self, recompute=False, **dataloader_kwargs):
         # If dataloaders were not already precomputed or if we want to recompute them to account
         # for changes in the global dataset
         if recompute or "train_dataloader" not in self.__dict__:
@@ -165,7 +246,7 @@ class Task:
         """Task hash is a hash of all RNA ids and node IDs in the dataset"""
         h = hashlib.new("sha256")
         if not self.in_memory:
-            raise ValueError("task id is only available (and tractable) for small, in-memory datasets")
+            return ""
         for rna in self.dataset.rnas:
             h.update(rna.name.encode("utf-8"))
             for nt in sorted(rna.nodes()):
@@ -185,12 +266,13 @@ class Task:
         if not os.path.exists(self.dataset_path) or not os.listdir(self.dataset_path) or self.recompute:
             print(">>> Saving dataset.")
             self.dataset.save(self.dataset_path, recompute=self.recompute)
-        with open(Path(self.root) / "train_idx.txt", "w") as idx:
-            [idx.write(str(ind) + "\n") for ind in self.train_ind]
-        with open(Path(self.root) / "val_idx.txt", "w") as idx:
-            [idx.write(str(ind) + "\n") for ind in self.val_ind]
-        with open(Path(self.root) / "test_idx.txt", "w") as idx:
-            [idx.write(str(ind) + "\n") for ind in self.test_ind]
+        if hasattr(self, "train_ind"):
+            with open(Path(self.root) / "train_idx.txt", "w") as idx:
+                [idx.write(str(ind) + "\n") for ind in self.train_ind]
+            with open(Path(self.root) / "val_idx.txt", "w") as idx:
+                [idx.write(str(ind) + "\n") for ind in self.val_ind]
+            with open(Path(self.root) / "test_idx.txt", "w") as idx:
+                [idx.write(str(ind) + "\n") for ind in self.test_ind]
         with open(Path(self.root) / "metadata.json", "w") as meta:
             json.dump(self.metadata, meta, indent=4)
 
@@ -200,19 +282,51 @@ class Task:
                 tid.write(self.task_id)
         print(">>> Done")
 
+    def to_csv(self, path: str):
+        """ Write a single CSV with all task data."""
+        rows = []
+        graph_level = self.metadata['graph_level']
+        for i, rna in enumerate(self.dataset):
+            if i in self.train_ind:
+                split = "train"
+            elif i in self.val_ind:
+                split = "val"
+            elif i in self.test_ind:
+                split = "test"
+            else:
+                raise IndexError
+
+            if graph_level:
+                target = rna['rna'].graph[self.target_var]
+            for node in rna['rna']:
+                if not graph_level:
+                    target = rna['rna'].nodes[node][self.target_var]
+                rows.append({"residue": node, "split": split, "target":
+                             target})
+                pass
+
+        pd.DataFrame(rows).to_csv(path)
+
     def load(self):
         """Load dataset and splits from disk."""
         # load splits
         print(">>> Loading precomputed dataset...")
-        self.train_ind = [int(ind) for ind in open(os.path.join(self.root, "train_idx.txt")).readlines()]
-        self.val_ind = [int(ind) for ind in open(os.path.join(self.root, "val_idx.txt")).readlines()]
-        self.test_ind = [int(ind) for ind in open(os.path.join(self.root, "test_idx.txt")).readlines()]
-
-        self.dataset = RNADataset(dataset_path=self.dataset_path, in_memory=self.in_memory, debug=self.debug)
+        self.dataset = RNADataset(
+            dataset_path=self.dataset_path, in_memory=self.in_memory, debug=self.debug, recompute_mapping=self.recompute
+        )
 
         with Path.open(Path(self.root) / "metadata.json") as meta:
             self.metadata = json.load(meta)
+        if (
+            os.path.exists(os.path.join(self.root, "train_idx.txt"))
+            and os.path.exists(os.path.join(self.root, "val_idx.txt"))
+            and os.path.exists(os.path.join(self.root, "test_idx.txt"))
+        ):
+            self.train_ind = [int(ind) for ind in open(os.path.join(self.root, "train_idx.txt")).readlines()]
+            self.val_ind = [int(ind) for ind in open(os.path.join(self.root, "val_idx.txt")).readlines()]
+            self.test_ind = [int(ind) for ind in open(os.path.join(self.root, "test_idx.txt")).readlines()]
 
+        self.dataset.features_computer = self.get_task_vars()
         return self.dataset, self.metadata, (self.train_ind, self.val_ind, self.test_ind)
 
     def __eq__(self, other):
@@ -221,7 +335,7 @@ class Task:
     def __repr__(self) -> str:
         return f"{self.__class__.__name__}()"
 
-    def describe(self, recompute=False):
+    def describe(self):
         """Get description of task dataset.
 
         Including dimensions needed for model initialization
@@ -230,18 +344,17 @@ class Task:
         Returns:
             dict: Contains dataset information and model dimensions
         """
-        if not recompute and "description" in self.metadata:
-            info = self.metadata["description"]
-        else:
-            print(">>> Computing description of task...")
-            self.get_split_loaders(recompute=False)
 
+        try:
+            self.metadata["dataset_size"]
+        except KeyError:
+            print(">>> Computing description of task...")
             # Get dimensions from first graph
             first_item = self.dataset[0]
-            compute_num_edge_attributes = "graph" in first_item
 
             first_node_map = {n: i for i, n in enumerate(sorted(first_item["rna"].nodes()))}
             first_features_dict = self.dataset.features_computer(first_item)
+            print(first_features_dict)
             first_features_array = first_features_dict["nt_features"][next(iter(first_node_map.keys()))]
             num_node_features = first_features_array.shape[0]
 
@@ -250,11 +363,10 @@ class Task:
             classes = set()
             unique_edge_attrs = set()  # only used with graphs
 
+
+            multi_label = self.metadata['multi_label']
             # Collect statistics from dataset
             for item in tqdm.tqdm(self.dataset):
-                if compute_num_edge_attributes:
-                    graph = item["graph"]
-                    unique_edge_attrs.update(graph.edge_attr.tolist())
                 node_map = {n: i for i, n in enumerate(sorted(item["rna"].nodes()))}
                 features_dict = self.dataset.features_computer(item)
                 if "nt_targets" in features_dict:
@@ -269,7 +381,7 @@ class Task:
                     y = features_dict["rna_targets"].clone().detach()
 
                 # In the multi_label case, a full binary matrix is better supported by both pytorch and sklearn
-                if not self.multi_label:
+                if not multi_label:
                     graph_classes = y.unique().tolist()
                 else:
                     graph_classes = torch.where(y > 0)[-1].unique().tolist()
@@ -280,7 +392,7 @@ class Task:
                     cls_int = int(cls)
                     if cls_int not in class_counts:
                         class_counts[cls_int] = 0
-                    if not self.multi_label:
+                    if not self.metadata['multi_label']:
                         class_counts[cls_int] += torch.sum(y == cls).item()
                     else:
                         class_counts[cls_int] += torch.sum(torch.where(y > 0)[-1] == cls).item()
@@ -291,12 +403,6 @@ class Task:
                 "dataset_size": len(self.dataset),
                 "class_distribution": class_counts,
             }
-            if compute_num_edge_attributes:
-                info["num_edge_attributes"] = len(unique_edge_attrs)
-            self.metadata["description"] = info
-            if self.save:
-                with open(Path(self.root) / "metadata.json", "w") as meta:
-                    json.dump(self.metadata, meta, indent=4)
 
         # Print description
         print("Dataset Description:")
@@ -307,14 +413,15 @@ class Task:
                 print("Class distribution:")
                 for cls in sorted(v.keys()):
                     print(f"\tClass {cls}: {v[cls]} {'nodes'}")
-        print()
         return info
 
     def create_dataset_from_list(self, rnas):
         """Computes an RNADataset object from the"""
         if self.in_memory:
+            print("in memory from list")
             dataset = RNADataset(rnas=rnas)
         else:
+            print("disk from list")
             dataset = RNADataset(dataset_path=self.dataset_path, rna_id_subset=rnas)
         return dataset
 
@@ -333,16 +440,14 @@ class Task:
 
 
 class ClassificationTask(Task):
-    def __init__(self, graph_level=False, num_classes=None, **kwargs):
+    def __init__(self, **kwargs):
         super().__init__(**kwargs)
-        self.num_classes = self.metadata["description"]["num_classes"] if num_classes is None else num_classes
-        self.graph_level = graph_level
 
     @property
     def dummy_model(self) -> torch.nn:
-        if self.graph_level:
-            return DummyGraphModel(num_classes=self.num_classes)
-        return DummyResidueModel(num_classes=self.num_classes)
+        if self.metadata['graph_level']:
+            return DummyGraphModel(num_classes=self.metadata['num_classes'])
+        return DummyResidueModel(num_classes=self.metadata['num_classes'])
 
     def dummy_inference(self):
         all_probs = []
@@ -367,7 +472,7 @@ class ClassificationTask(Task):
                 labels = tonumpy(labels)
 
                 # split predictions per RNA if residue level
-                if not self.graph_level:
+                if not self.metadata['graph_level']:
                     cumulative_sizes = tuple(tonumpy(graph.ptr))
                     probs = [
                         probs[start:end]
@@ -397,7 +502,7 @@ class ClassificationTask(Task):
                 all_preds.extend(preds)
                 all_labels.extend(labels)
 
-        if self.graph_level:
+        if self.metadata['graph_level']:
             all_probs = np.stack(all_probs)
             all_preds = np.stack(all_preds)
             all_labels = np.stack(all_labels)
@@ -412,13 +517,16 @@ class ClassificationTask(Task):
                 average="binary" if self.num_classes == 2 else "macro",
             ),
         }
-        if not self.multi_label:
+        if not self.metadata['multi_label']:
             one_metric["mcc"] = matthews_corrcoef(labels, preds)
+            one_metric["balanced_accuracy"] = balanced_accuracy_score(labels, preds)
+        if self.metadata['multi_label']:
+            one_metric["jaccard"] = jaccard_score(labels, preds, average="macro")
         try:
             one_metric["auc"] = roc_auc_score(
                 labels,
                 probs,
-                average=None if self.num_classes == 2 else "macro",
+                average=None if self.metadata['num_classes'] == 2 else "macro",
                 multi_class="ovo",
             )
         except:
@@ -426,7 +534,7 @@ class ClassificationTask(Task):
         return one_metric
 
     def compute_metrics(self, all_preds, all_probs, all_labels):
-        if self.graph_level:
+        if self.metadata['graph_level']:
             return self.compute_one_metric(all_preds, all_probs, all_labels)
         # Here we have a list of preds [(n1,), (n2,)...] for each residue in each RNA
         # Either compute the overall flattened results, or aggregate by system
@@ -454,10 +562,12 @@ class ClassificationTask(Task):
 
 
 class ResidueClassificationTask(ClassificationTask):
-    def __init__(self, **kwargs):
-        super().__init__(graph_level=False, **kwargs)
+    def __init__(self, additional_metadata=None, **kwargs):
+        meta = {'graph_level': False, **additional_metadata}
+        super().__init__(additional_metadata=meta, **kwargs)
 
 
 class RNAClassificationTask(ClassificationTask):
-    def __init__(self, **kwargs):
-        super().__init__(graph_level=True, **kwargs)
+    def __init__(self, additional_metadata=None, **kwargs):
+        meta = {'graph_level': True, **additional_metadata}
+        super().__init__( additional_metadata=meta, **kwargs)
