@@ -15,7 +15,7 @@ import pandas as pd
 import requests
 from Bio.PDB.PDBList import PDBList
 
-ZENODO_RECORD = "14635910"
+ZENODO_RECORD = "14930728"
 
 
 def multigraph_to_simple(g: nx.MultiDiGraph) -> nx.DiGraph:
@@ -102,7 +102,7 @@ def load_graph(filename, multigraph=False):
 
     :return: networkx DiGraph object
     """
-    if filename.endswith("json"):
+    if str(filename).endswith("json"):
         graph = load_json(filename)
     elif filename.endswith("p"):
         pickled = pickle.load(open(filename, "rb"))
@@ -149,7 +149,7 @@ def get_all_existing(dataset_path: os.PathLike, all_rnas: list[str] | None = Non
     :param all_rnas: list of RNA names to search for (e.g. ``'1aju'`` will match ``'1aju.json'`` in ``dataset_path``.
     :return: List of filenames in ``dataset_path``
     """
-    special_names = {"metadata.json", "mapping.json", "distances.npz"}
+    special_names = {"metadata.json", "bidict.json", "distances.npz"}
     potential_rnas = [file_name for file_name in sorted(os.listdir(dataset_path)) if file_name not in special_names]
     _, extension = get_name_extension(potential_rnas[0])
 
@@ -253,7 +253,7 @@ def download(url, path=None, overwrite=True, retries=5, verify_ssl=True, log=Tru
     return fname
 
 
-def download_name_generator(version="1.0.0", redundancy="nr", annotated=False, record="14325118", debug=False):
+def download_name_generator(version="2.0.2", redundancy="nr", annotated=False, record="14325118", debug=False):
     """This returns the zenodo URL given dataset choices.
 
     :param version: Which data version to use.
@@ -274,7 +274,7 @@ def download_name_generator(version="1.0.0", redundancy="nr", annotated=False, r
 
 def download_graphs(
     redundancy="nr",
-    version="1.0.0",
+    version="2.0.2",
     annotated=False,
     chop=False,
     overwrite=False,
@@ -342,7 +342,7 @@ def download_graphs(
 
 def available_pdbids(
     graph_dir=None,
-    version="1.0.0",
+    version="2.0.2",
     chop=False,
     annotated=False,
     redundancy="nr",
